#ifndef MAINFRAME_H
#define MAINFRAME_H

#include <wx/wx.h>
#include <wx/datetime.h>
#include "Quiz.h"

class PanelPainter : public wxEvtHandler
{
private:
    Quiz *quiz;
    wxPanel *panel;
public:
    PanelPainter (Quiz *q, wxPanel *p) : quiz(q), panel (p) {}
    
protected:
    DECLARE_EVENT_TABLE()
    void OnPaint (wxPaintEvent &event);
};

class MainFrame : public wxFrame
{
private:
    enum {
	    menuNew = 1000,
	    menuRestart,
	    menuOptions,
	    menuExit,
	    menuStats,
	    menuAbout,
	    txtEnter,
	    pnlPaint
    };
    
    DECLARE_EVENT_TABLE()
    
    wxTextCtrl *txtInput;
    wxTextCtrl *txtOutput;
    wxPanel *binPanel;
    wxStaticText *lblResult1, *lblResult2;
    
    // for statistics keeping:
    wxDateTime starttime;
    int startcounter;
    
    Quiz quiz;
	std::string lessonPath;
    
    void OnMenuNew (wxCommandEvent &event);
    void OnMenuRestart (wxCommandEvent &event);
    void OnMenuOptions (wxCommandEvent &event);
    void OnMenuExit (wxCommandEvent &event);
    void OnMenuStats (wxCommandEvent &event);
    void OnMenuAbout (wxCommandEvent &event);
    void OnTxtEnter (wxCommandEvent &event);
    void OnClose (wxCloseEvent &event);

    void DrawBins();
    
    // called at beginning, loads settings file and
    // optionally loads stored quiz
    void init(); 
    
    // pop up a file selection dialog and let the user
    // select a lesson.
    void selectLesson();

    // let Quiz pick next word, and display the question
    void nextWord();
    
    // run the answer through quiz to see if it is correct,
    // and display the results. If the quiz is over, show the final results and clear the frame
    void checkAnswer();
    
    // pop up the results dialog when the quiz is finished.
    void showResults();
    
    // called whenever there is no question currently to display
    void clearFrame();
    
    // called to write progress to log
    void writeLog();
    
    // called to start measuring time
    void beginLog();
public:
	MainFrame (const wxChar *title, int xpos, int ypos, int width, int height, std::string lp);
    ~MainFrame();
};

#endif
