#include "Word.h"
#include <assert.h>
#include <iostream>

using namespace std;

bool WriteInt (fstream &f, int output)
{
    f.write ((char*)(&output), sizeof (output));
    return f.good();
}

int ReadInt (fstream &f)
{
    int result;
    f.read ((char *)(&result), sizeof (result));
    return result;
}

bool WriteString (fstream &f, const std::string &output)
{
    unsigned int size = output.size();    
    f.write ((char*)(&size), sizeof (size));
    f.write (output.c_str(), size);
    return f.good();
}

string ReadString (fstream &f)
{
    int size;    
    char *buffer = NULL;
    bool error = false;
    if (!error)
    {
        f.read ((char*)(&size), sizeof (size));
        if (!f.good()) error = true;
        assert (size <= 60000); // sanity check
    }
    if (!error) 
    {   buffer = new char[size + 1];
        if (!buffer) error = true;
    }
    if (!error) 
    {
        buffer[size] = 0; // terminate with null character
        f.read (buffer, size);
        if (!f.good()) error = true;
    }  
    string result = "";
    if (!error)
    {
        result = string(buffer);
    }
    if (buffer) delete [] buffer;
    return result;
}

Word::Word (string &q, string &a, int _dir, int _pair) :
    left (1), bin (0), howSoon (-1), quizCount (0), 
    correctCount (0), question (q), answer (a), wrongAnswers(),
    dir (_dir), pair (_pair)
{
}

bool Word::Deserialize (fstream &f)
{
	left = ReadInt (f);
	bin = ReadInt (f);
	howSoon = ReadInt (f);
	quizCount = ReadInt (f);
	correctCount = ReadInt (f);
	question = ReadString (f);
	answer = ReadString (f);
	int size = ReadInt (f);
	for (int i = 0; i < size; ++i)
	{
		string a = ReadString (f);
		int c = ReadInt (f);
		wrongAnswers[a] = c;
	}
	dir = ReadInt (f);
	pair = ReadInt (f);
	return true;
}

bool Word::Serialize (fstream &f) const
{
    WriteInt (f, left);
    WriteInt (f, bin);
    WriteInt (f, howSoon);
    WriteInt (f, quizCount);
    WriteInt (f, correctCount);
    WriteString (f, question);
    WriteString (f, answer);    
    WriteInt (f, wrongAnswers.size());
    map <string, int, less<string> >::const_iterator i;
    for (i = wrongAnswers.begin(); i != wrongAnswers.end(); ++i)
    {
        WriteString (f, i->first);
        WriteInt (f, i->second);
    }
    WriteInt (f, dir);
    WriteInt (f, pair);
    return true;
}

bool Word::CompareAnswer (string &a, int counter, int (&binCount)[MAXBINS])
{
    bool result;
    result = (a.compare (answer) == 0);
    if (result)
    {
        quizCount++;
        correctCount++;
        if (--left == 0)
        {
            if (bin < MAXBINS-1)
            {
                binCount[bin]--;
                bin++;
                binCount[bin]++;
            }
            howSoon = -1;
            left = 1;
        }
        else
            howSoon = counter;
    }
    else
    {
        quizCount++;
        howSoon = counter;
        left = 2;
        
        // keep statistics about wrong answers.
        map <string, int, less <string> >::iterator i = 
            wrongAnswers.find (a);
        int count;
        if (i != wrongAnswers.end()) count = (i->second) + 1; else count = 1;
        wrongAnswers[a] = count;
    }
    return result;
}
