#ifndef WORD_H
#define WORD_H

#include <string>
#include <map>
#include <fstream>

const int MAXBINS = 10;

// for writing strings to binary streams
bool WriteString (std::fstream &f, const std::string &output);

bool WriteInt (std::fstream &f, int output);

int ReadInt (std::fstream &f);

// for reading strings from binary streams.
std::string ReadString (std::fstream &f);

typedef std::map <std::string, int, std::less<std::string> > WrongAnswersType;

class Word
{
private:
	int left; // # of times to be answered before moving to next bin, starts at one
    int bin;
    int howSoon;
    int quizCount; // # of times asked
    int correctCount; // # of times correct
    std::string question, answer;
    WrongAnswersType wrongAnswers;
    int dir; // direction of this question 1->0 or 0->1
    int pair; // order in lesson file.
public:
    Word (std::string &q, std::string &a, int _dir, int _pair);
    const std::string &GetQuestion () const { return question; }
    const std::string &GetAnswer () const { return answer; }
    bool CompareAnswer(std::string &a, int counter, int (&binCount)[MAXBINS]);
    int GetDir () const { return dir; }
    int GetBin () const { return bin; }
    int GetHowSoon () const { return howSoon; }
    int GetQuizCount () const { return quizCount; }
    const WrongAnswersType &GetWrongAnswers () { return wrongAnswers; }
	bool Serialize (std::fstream &f) const;    
    bool Deserialize (std::fstream &f);
};

#endif

